@@ -3,6 +3,7 @@ __all__ = ["ExperimentIterableDataset"]
 import os
 import pandas as pd
 
+from re import findall
 from typing import Union, Optional
 from warnings import warn
 from ibm_watson_machine_learning.helpers.connections.flight_service import (
@@ -20,6 +21,10 @@ except ImportError:
     IterableDataset = object
 # --- end note
 
+DEFAULT_SAMPLE_SIZE_LIMIT = 1073741824  # 1GB in Bytes is verified later by _set_sample_size_limit
+DEFAULT_REDUCED_SAMPLE_SIZE_LIMIT = 104857600  # 100MB in bytes
+DEFAULT_SAMPLING_TYPE = SamplingTypes.FIRST_N_RECORDS
+
 
 class ExperimentIterableDataset(IterableDataset):
     """
@@ -69,15 +74,15 @@ class ExperimentIterableDataset(IterableDataset):
         connection: "DataConnection",
         experiment_metadata: dict = None,
         enable_sampling: bool = True,
-        sample_size_limit: int = 1073741824,  # 1GB in Bytes is verified later by _set_sample_size_limit,
-        sampling_type: str = SamplingTypes.FIRST_N_RECORDS,
+        sample_size_limit: int = DEFAULT_SAMPLE_SIZE_LIMIT,
+        sampling_type: str = DEFAULT_SAMPLING_TYPE,
         binary_data: bool = False,
         number_of_batch_rows: int = None,
         stop_after_first_batch: bool = False,
         **kwargs,
     ):
         super().__init__()
-        self._set_sample_size_limit_()
+        self._set_sample_size_limit(sample_size_limit)
         self.experiment_metadata = experiment_metadata
         self.enable_sampling = enable_sampling
         self._wml_client = getattr(connection, "_wml_client", None)
@@ -127,12 +132,12 @@ class ExperimentIterableDataset(IterableDataset):
                 label=self.experiment_metadata.get("prediction_column"),
                 learning_type=self.experiment_metadata.get("prediction_type"),
                 params=self.experiment_metadata,
-                project_id=self.experiment_metadata.get("project_id"),
-                space_id=self.experiment_metadata.get("space_id"),
+                project_id=self.experiment_metadata.get("project_id", getattr(self._wml_client, 'default_project_id', None)),
+                space_id=self.experiment_metadata.get("space_id", getattr(self._wml_client, 'default_space_id', None)),
                 asset_id=dict_connection.get("location", {}).get("id"),
                 connection_id=dict_connection.get("connection", {}).get("id"),
                 data_location=dict_connection,
-                data_size_limit=sample_size_limit,
+                data_size_limit=self.sample_size_limit,
                 flight_parameters=kwargs.get("flight_parameters", {}),
                 extra_interaction_properties=kwargs.get("extra_interaction_properties",{}),
                 fallback_to_one_connection=kwargs.get(
@@ -140,7 +145,8 @@ class ExperimentIterableDataset(IterableDataset):
                 ),
                 number_of_batch_rows=number_of_batch_rows,
                 stop_after_first_batch=stop_after_first_batch,
-                _wml_client=kwargs.get('_wml_client')
+                _wml_client=kwargs.get('_wml_client'),
+                return_subsampling_stats=kwargs.get("_return_subsampling_stats", False)
             )
 
         else:
@@ -187,18 +193,20 @@ class ExperimentIterableDataset(IterableDataset):
         else:
             return False
 
-    def _set_sample_size_limit_(self):
-        """Set Sample Size Limit based on T-Shirt size if code is run on training pod
+    def _set_sample_size_limit(self, sample_size_limit):
+        """If non-default value of sample_size_limit was not passed,
+        set Sample Size Limit based on T-Shirt size if code is run on training pod:
         For memory < 32 (T-Shirts: XS,S, M) default is 100MB,
         For memory >= 32 or runs outside pod default is 1GB
         """
-        import re
-
-        size_mem = int(re.findall(r'[0-9]+', os.environ.get('MEM', '32'))[0])
-        if size_mem >= 32:
-            self.sample_size_limit = 1073741824     # 1GB in bytes
+        if sample_size_limit == DEFAULT_SAMPLE_SIZE_LIMIT:
+            size_mem = int(findall(r'[0-9]+', os.environ.get('MEM', '32'))[0])
+            if size_mem >= 32:
+                self.sample_size_limit = DEFAULT_SAMPLE_SIZE_LIMIT     # 1GB in bytes
+            else:
+                self.sample_size_limit = DEFAULT_REDUCED_SAMPLE_SIZE_LIMIT      # 100MB in bytes
         else:
-            self.sample_size_limit = 104857600      # 100MB in bytes
+            self.sample_size_limit = sample_size_limit
 
     def write(
         self, data: Optional[pd.DataFrame] = None, file_path: Optional[str] = None
