@@ -250,13 +250,19 @@ class BaseDeployment(ABC):
                     data_connection = None
                 # --- end note
 
+                try:
+                    auto_pipelines_parameters = wml_pipeline_details.get('entity', {}).get('document', {}).get('pipelines', [])[0].get('nodes', [])[0].get('parameters')
+                except:
+                    auto_pipelines_parameters = None
+
                 artifact_name, model_props = prepare_auto_ai_model_to_publish_normal_scenario(
                     pipeline_model=model,
                     run_params=run_params,
                     run_id=kwargs['experiment_run_id'],
                     wml_client=self._source_workspace.wml_client,
                     space_id=self._target_workspace.space_id,
-                    result_reference=data_connection
+                    result_reference=data_connection,
+                    auto_pipelines_parameters=auto_pipelines_parameters
                 )
 
             deployment_details = self._deploy(
@@ -404,7 +410,8 @@ class BaseDeployment(ABC):
                             result_connection=optimizer._result_client[0],
                             cos_client=optimizer._result_client[1],
                             run_params=run_details,
-                            space_id=self._target_workspace.space_id
+                            space_id=self._target_workspace.space_id,
+                            auto_pipelines_parameters=optimizer.get_params()
                         )
 
                         deployment_details = self._deploy(
@@ -426,7 +433,8 @@ class BaseDeployment(ABC):
                             run_params=run_params,
                             run_id=run_id,
                             wml_client=self._source_workspace.wml_client,
-                            space_id=self._target_workspace.space_id
+                            space_id=self._target_workspace.space_id,
+                            auto_pipelines_parameters=optimizer.get_params()
                         )
 
                         deployment_details = self._deploy(
