@@ -754,9 +754,8 @@ class Deployments(WMLResource):
 
         #########################################
 
-    @staticmethod
     @docstring_parameter({'str_type': STR_TYPE_NAME})
-    def get_download_url(deployment_details):
+    def get_download_url(self, deployment_details):
         """
         Get deployment_download_url from deployment details.
 
@@ -777,6 +776,12 @@ class Deployments(WMLResource):
 
             >>> deployment_url = client.deployments.get_download_url(deployment)
         """
+        if self._client.ICP_PLATFORM_SPACES:
+            raise WMLClientError(u'Downloading virtual deployment is no longer supported in Cloud Pak for Data, versions 3.5 and later.')
+
+        if self._client.CLOUD_PLATFORM_SPACES:
+            raise WMLClientError(u'Downloading virtual deployment is no longer supported in Cloud Pak for Data as a Service.')
+
         Deployments._validate_type(deployment_details, u'deployment_details', dict, True)
         try:
             virtual_deployment_detaails = deployment_details.get(u'entity').get(u'status').get(u'virtual_deployment_downloads')
@@ -796,6 +801,7 @@ class Deployments(WMLResource):
     def download(self, virtual_deployment_uid, filename=None):
         """
         Downloads file deployment of specified deployment Id. Currently supported format is Core ML.
+        Supported only for IBM Cloud Pak® for Data 3.0 and below.
 
         **Parameters**
 
