@@ -29,6 +29,7 @@ from ibm_watson_machine_learning.utils.autoai.utils import check_dependencies_ve
     translate_imputation_string_strategy_to_enum, translate_estimator_string_to_enum, \
     translate_batched_estimator_string_to_enum
 from ibm_watson_machine_learning.workspace import WorkSpace
+from ibm_watson_machine_learning.wml_client_error import ForbiddenActionForGitBasedProject
 from .engines import WMLEngine
 from .optimizers import LocalAutoPipelines, RemoteAutoPipelines
 from .runs import AutoPipelinesRuns, LocalAutoPipelinesRuns
@@ -116,6 +117,8 @@ class AutoAI(BaseExperiment):
             self.runs = AutoPipelinesRuns(engine=WMLEngine(self._workspace))
             self.runs._workspace = self._workspace
 
+        self._block_autoai_on_git_based_project()
+
         self._init_estimator_enums()
 
         self._20_class_limit_removal_test = False
@@ -185,12 +188,14 @@ class AutoAI(BaseExperiment):
                   daub_give_priority_to_runtime: float = None,
                   fairness_info: dict = None,
                   sampling_type: 'SamplingTypes' = None,
+                  sample_size_limit: int = None,
                   n_parallel_data_connections: int = None,
+                  number_of_batch_rows: int = None,
                   categorical_imputation_strategy: ImputationStrategy = None,
                   numerical_imputation_strategy: ImputationStrategy = None,
                   numerical_imputation_value: float = None,
                   imputation_threshold: float = None,
-                  retrain_on_holdout: int = True,
+                  retrain_on_holdout: bool = None,
                   categorical_columns: list = None,
                   numerical_columns: list = None,
                   test_data_csv_separator: Union[List[str], str] = ',',
@@ -372,6 +377,13 @@ class AutoAI(BaseExperiment):
             Default is SamplingTypes.FIRST_N_RECORDS.
             Supported only for IBM Cloud Pak® for Data 4.0.1 and above.
 
+        sample_size_limit: int, optional
+            The size of sample upper bound (in bytes). The default value is 1GB.
+            Supported only for IBM Cloud Pak® for Data 4.5 and above.
+
+        number_of_batch_rows: int, optional
+            Number of rows to read in each batch when reading from flight connection.
+
         test_data_csv_separator: Union[List[str], str], optional
             The separator, or list of separators to try for separating
             columns in a CSV user-defined holdout/test file.  Not used if the file_name is not a CSV file.
@@ -441,18 +453,21 @@ class AutoAI(BaseExperiment):
             include_batched_ensemble_estimators = kwargs.pop('global_stage_include_batched_ensemble_estimators')  # TODO to be removed
 
         # TS exogenous, to be removed
-        supporting_features = None
+        feature_columns = None
         pipeline_types = None
-        supporting_features_at_forecasting = None
+        supporting_features_at_forecast = None
 
-        if 'supporting_features' in kwargs:
-            supporting_features = kwargs.pop('supporting_features')
+        if 'feature_columns' in kwargs:
+            feature_columns = kwargs.pop('feature_columns')
 
         if 'pipeline_types' in kwargs:
             pipeline_types = kwargs.pop('pipeline_types')
 
-        if 'supporting_features_at_forecasting' in kwargs:
-            supporting_features_at_forecasting = kwargs.pop('supporting_features_at_forecasting')
+        if 'supporting_features_at_forecast' in kwargs:
+            supporting_features_at_forecast = kwargs.pop('supporting_features_at_forecast')
+
+        if prediction_type != PredictionType.FORECASTING and retrain_on_holdout is None:
+            retrain_on_holdout = True
 
         # Deprecation of excel_sheet as number:
         if isinstance(excel_sheet, int):
@@ -627,6 +642,8 @@ class AutoAI(BaseExperiment):
                 obm_pod_version=kwargs.get('obm_pod_version', None),
                 fairness_info=fairness_info,
                 sampling_type=sampling_type,
+                sample_size_limit=sample_size_limit,
+                number_of_batch_rows=number_of_batch_rows,
                 n_parallel_data_connections=n_parallel_data_connections,
                 categorical_imputation_strategy=categorical_imputation_strategy,
                 numerical_imputation_strategy=numerical_imputation_strategy,
@@ -635,9 +652,9 @@ class AutoAI(BaseExperiment):
                 retrain_on_holdout=retrain_on_holdout,
                 numerical_columns=numerical_columns,
                 categorical_columns=categorical_columns,
-                supporting_features=supporting_features,
+                feature_columns=feature_columns,
                 pipeline_types=pipeline_types,
-                supporting_features_at_forecasting=supporting_features_at_forecasting,
+                supporting_features_at_forecast=supporting_features_at_forecast,
                 test_data_csv_separator=test_data_csv_separator,
                 test_data_excel_sheet=test_data_excel_sheet,
                 test_data_encoding=test_data_encoding,
@@ -655,3 +672,17 @@ class AutoAI(BaseExperiment):
             self.ClassificationAlgorithms = ClassificationAlgorithms
             self.RegressionAlgorithms = RegressionAlgorithms
             self.ForecastingAlgorithms = ForecastingAlgorithms
+
+    def _block_autoai_on_git_based_project(self):
+        """
+        Raises ForbiddenActionForGitBasedProject error for AutoAI experiments on git based project.
+        It can be disabled by setting environment variable ENABLE_AUTOAI to 'true'
+        """
+        from os import environ
+
+        if self._workspace:
+            if getattr(self._workspace.wml_client, 'project_type', None) == 'local_git_storage' \
+                    and environ.get('ENABLE_AUTOAI', 'false').lower() == 'false':
+                raise ForbiddenActionForGitBasedProject(reason="Creating AutoAI experiment is not supported for git based project.")
+
+
