@@ -150,10 +150,10 @@ class WMLEngine(BaseEngine):
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
                     'text_columns'] = self._auto_pipelines_parameters.get('text_columns_names')
                 # end of Text transformer parameters
-                if self._auto_pipelines_parameters.get('retrain_on_holdout'):
-                    self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                        'pipelines'][0]['nodes'][0]['parameters']['optimization'][
-                        'retrain_on_holdout'] = self._auto_pipelines_parameters.get('retrain_on_holdout')
+            if self._auto_pipelines_parameters.get('retrain_on_holdout'):
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['optimization'][
+                    'retrain_on_holdout'] = self._auto_pipelines_parameters.get('retrain_on_holdout')
         # end of section
 
         # pass fairness info for Cloud or CP4D 4.5
@@ -165,36 +165,40 @@ class WMLEngine(BaseEngine):
         # end of section
 
         # note: Additional parameters for benchmark subsampling with data detector
-        if self._wml_client.ICP and (self._wml_client.ICP_40 or self._wml_client.ICP_45):
-            if self._auto_pipelines_parameters.get('sampling_type') is not None:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                    'pipelines'][0]['nodes'][0]['parameters'][
-                    'sampling_type'] = self._auto_pipelines_parameters.get('sampling_type')
+        if self._auto_pipelines_parameters.get('sampling_type') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'sampling_type'] = self._auto_pipelines_parameters.get('sampling_type')
 
-            if self._auto_pipelines_parameters.get('n_parallel_data_connections') is not None:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                    'pipelines'][0]['nodes'][0]['parameters'][
-                    'n_parallel_data_connections'] = self._auto_pipelines_parameters.get('n_parallel_data_connections')
+        if self._auto_pipelines_parameters.get('sample_size_limit') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'sample_size_limit'] = self._auto_pipelines_parameters.get('sample_size_limit')
 
-            if self._auto_pipelines_parameters.get('calculate_data_metrics') is not None:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                    'pipelines'][0]['nodes'][0]['parameters'][
-                    'calculate_data_metrics'] = self._auto_pipelines_parameters.get('calculate_data_metrics')
+        if self._auto_pipelines_parameters.get('n_parallel_data_connections') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'n_parallel_data_connections'] = self._auto_pipelines_parameters.get('n_parallel_data_connections')
 
-            if self._auto_pipelines_parameters.get('logical_batch_size') is not None:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                    'pipelines'][0]['nodes'][0]['parameters'][
-                    'logical_batch_size'] = self._auto_pipelines_parameters.get('logical_batch_size')
+        if self._auto_pipelines_parameters.get('calculate_data_metrics') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'calculate_data_metrics'] = self._auto_pipelines_parameters.get('calculate_data_metrics')
 
-            if self._auto_pipelines_parameters.get('metrics_on_logical_batch') is not None:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                    'pipelines'][0]['nodes'][0]['parameters'][
-                    'metrics_on_logical_batch'] = self._auto_pipelines_parameters.get('metrics_on_logical_batch')
+        if self._auto_pipelines_parameters.get('logical_batch_size') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'logical_batch_size'] = self._auto_pipelines_parameters.get('logical_batch_size')
 
-            if self._auto_pipelines_parameters.get('batch_size') is not None:
-                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                    'pipelines'][0]['nodes'][0]['parameters'][
-                    'batch_size'] = self._auto_pipelines_parameters.get('batch_size')
+        if self._auto_pipelines_parameters.get('metrics_on_logical_batch') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'metrics_on_logical_batch'] = self._auto_pipelines_parameters.get('metrics_on_logical_batch')
+
+        if self._auto_pipelines_parameters.get('number_of_batch_rows') is not None:
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'][0]['parameters'][
+                'number_of_batch_rows'] = self._auto_pipelines_parameters.get('number_of_batch_rows')
         # --- end note
 
         if self._auto_pipelines_parameters.get('prediction_type') == PredictionType.FORECASTING:
@@ -220,10 +224,10 @@ class WMLEngine(BaseEngine):
                 self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
                     'gap_len'] = self._auto_pipelines_parameters['backtest_gap_length']
-            if self._auto_pipelines_parameters['supporting_features']:
+            if self._auto_pipelines_parameters['feature_columns']:
                 self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
-                    'feature_columns'] = self._auto_pipelines_parameters['supporting_features']
+                    'feature_columns'] = self._auto_pipelines_parameters['feature_columns']
             if self._auto_pipelines_parameters['pipeline_types']:
                 self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
@@ -232,13 +236,17 @@ class WMLEngine(BaseEngine):
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
                     'customized_pipelines'] = [pipeline.value for pipeline in
                                                self._auto_pipelines_parameters['pipeline_types']]
-            if self._auto_pipelines_parameters['supporting_features_at_forecasting'] != None:
+            if self._auto_pipelines_parameters['supporting_features_at_forecast'] != None:
                 self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                     'pipelines'][0]['nodes'][0]['parameters']['optimization'][
-                    'future_exogenous_available'] = self._auto_pipelines_parameters['supporting_features_at_forecasting']
+                    'future_exogenous_available'] = self._auto_pipelines_parameters['supporting_features_at_forecast']
             self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                 'pipelines'][0]['nodes'][0]['parameters'][
                 'compute_pipeline_notebooks_flag'] = self._auto_pipelines_parameters.get('notebooks', False)
+            if self._auto_pipelines_parameters.get('retrain_on_holdout'):
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['parameters']['optimization'][
+                    'retrain_on_holdout'] = self._auto_pipelines_parameters.get('retrain_on_holdout')
         else:
             self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
                 'pipelines'][0]['nodes'][0]['parameters']['optimization'][
@@ -706,7 +714,7 @@ class WMLEngine(BaseEngine):
                                     progress_bar.close()
                                     break
                                 elif 'AutoAI model computed' not in text:
-                                    prepared_text = text.split(': ')[-1].split("global:")[-1]
+                                    prepared_text = text.split('Node automl: ')[-1].split("global:")[-1].split(" Message: ")[-1]
                                     max_len = 40
 
                                     if len(prepared_text) > max_len:
@@ -1052,7 +1060,8 @@ class WMLEngine(BaseEngine):
                 pipeline_name=pipeline_name,
                 load_pipelines=True,
                 store=persist,
-                wml_client=self._wml_client
+                wml_client=self._wml_client,
+                auto_pipelines_parameters=self._auto_pipelines_parameters
             )
 
         except LibraryNotCompatible as e:
