@@ -158,15 +158,17 @@ class RemoteAutoPipelines(BaseAutoPipelines):
                  test_data_excel_sheet: Union[str, int] = None,
                  test_data_encoding: str = 'utf-8',
                  sampling_type=None,
+                 sample_size_limit=None,
+                 number_of_batch_rows=None,
                  categorical_imputation_strategy=None,
                  numerical_imputation_strategy=None,
                  numerical_imputation_value=None,
                  imputation_threshold=None,
                  fairness_info: dict = None,
                  retrain_on_holdout: bool = True,
-                 supporting_features: List[str] = None,
+                 feature_columns: List[str] = None,
                  pipeline_types: List['ForecastingPipelineTypes'] = None,
-                 supporting_features_at_forecasting: bool = None,
+                 supporting_features_at_forecast: bool = None,
                  **kwargs):
 
         if data_join_graph is not None:
@@ -211,6 +213,8 @@ class RemoteAutoPipelines(BaseAutoPipelines):
             'daub_give_priority_to_runtime': daub_give_priority_to_runtime,
             'text_columns_names': text_columns_names,
             'sampling_type': sampling_type,
+            'sample_size_limit': sample_size_limit,
+            'number_of_batch_rows': number_of_batch_rows,
             'n_parallel_data_connections': n_parallel_data_connections,
             'test_data_csv_separator': test_data_csv_separator,
             'test_data_excel_sheet': test_data_excel_sheet,
@@ -220,9 +224,9 @@ class RemoteAutoPipelines(BaseAutoPipelines):
             'numerical_imputation_value': numerical_imputation_value,
             'imputation_threshold': imputation_threshold,
             'retrain_on_holdout': retrain_on_holdout,
-            'supporting_features': supporting_features,
+            'feature_columns': feature_columns,
             'pipeline_types': pipeline_types,
-            'supporting_features_at_forecasting': supporting_features_at_forecasting
+            'supporting_features_at_forecast': supporting_features_at_forecast
         }
 
         if fairness_info:
