@@ -30,8 +30,9 @@ from pandas import concat, DataFrame, read_csv
 from ibm_watson_machine_learning.utils.autoai.utils import try_load_dataset, try_load_tar_gz, try_import_autoai_libs
 from ibm_watson_machine_learning.utils.autoai.enums import DataConnectionTypes
 from ibm_watson_machine_learning.utils.autoai.errors import CannotReadSavedRemoteDataBeforeFit, WrongAssetType
-from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure, WMLClientError
+from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure, WMLClientError, DataStreamError
 from ibm_watson_machine_learning.utils.utils import is_lib_installed
+from ibm_watson_machine_learning.data_loaders.datasets.experiment import DEFAULT_SAMPLING_TYPE, DEFAULT_SAMPLE_SIZE_LIMIT
 
 if TYPE_CHECKING:
     from ibm_boto3 import resource
@@ -220,9 +221,11 @@ class BaseDataConnection(ABC):
 
     def _check_if_connection_asset_is_s3(self) -> bool:
         try:
-            # fast return true if the connection is an instance of S3Connection and non s3 type
+            # fast return true if the connection is an instance of S3Connection and non s3 type, or its attribute
+            # "cos_type" takes True
             from .connections import S3Connection
-            if isinstance(self.connection, S3Connection) and self.type != 's3':
+            if (isinstance(self.connection, S3Connection) and self.type != 's3') or (
+                    self.connection is not None and getattr(self.connection, 'cos_type', False)):
                 return True
 
             if self.type == 'data_asset':
@@ -386,13 +389,15 @@ class BaseDataConnection(ABC):
                         self.location.bucket = connection_details['entity']['properties']['bucket']
                         self.location.path = attachment_details['connection_path']
 
+                    self.location.id = None
+                self.connection.cos_type = True
                 return True
 
             else:
                 return False
 
         except Exception as e:
-            if os.environ.get('USER_ACCESS_TOKEN') is None:
+            if os.environ.get('USER_ACCESS_TOKEN') is None and os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE') is None:
                 raise e
 
             else:
@@ -427,7 +432,7 @@ class BaseDataConnection(ABC):
                         raise NotImplementedError(f"This functionality can be run only on Watson Studio.")
 
         except Exception as e:
-            if os.environ.get('USER_ACCESS_TOKEN') is None:
+            if os.environ.get('USER_ACCESS_TOKEN') is None and os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE') is None:
                 raise e
             else:
                 return False  # if we are in WS, ignore this check even if there was some error
@@ -717,7 +722,6 @@ class BaseDataConnection(ABC):
             raise CannotReadSavedRemoteDataBeforeFit()
 
         buffer = None
-
         # Note: workaround with volumes API as connections API changes data format
         try:
             connection_details = self._wml_client.connections.get_details(connection_id)
@@ -820,7 +824,7 @@ class BaseDataConnection(ABC):
         else:
             try:
                 from project_lib import Project
-                token = os.environ.get('USER_ACCESS_TOKEN')
+                token = self._get_token_from_environment()
 
                 if token is None:
                     raise NotImplementedError(
@@ -868,7 +872,13 @@ class BaseDataConnection(ABC):
                                            read_to_file: str = None,
                                            flight_parameters: dict = None,
                                            headers: dict = None,
-                                           number_of_batch_rows: int = None):
+                                           number_of_batch_rows: int = None,
+                                           sample_size_limit: int = DEFAULT_SAMPLE_SIZE_LIMIT,
+                                           sampling_type: str = DEFAULT_SAMPLING_TYPE,
+                                           return_data_as_iterator: bool = False,
+                                           enable_sampling: bool = True,
+                                           _return_subsampling_stats: bool = False):
+
         is_lib_installed(lib_name='pyarrow', minimum_version='3.0.0', install=True)
 
         # import the class only if flight scenario is enabled - do not import it in main import section
@@ -891,21 +901,50 @@ class BaseDataConnection(ABC):
 
         experiment_metadata.update(self.auto_pipeline_params)
 
-        iterable_dataset = ExperimentIterableDataset(
+        experiment_iterable_dataset_setup_parameters = dict(
             connection=dict_connection,
-            enable_sampling=not binary,  # don't use sampling to read binary
+            enable_sampling=enable_sampling if not binary else False,  # don't use sampling to read binary
             experiment_metadata=experiment_metadata,
             binary_data=binary,
             read_to_file=read_to_file,
             flight_parameters=flight_parameters if flight_parameters is not None else {},
             fallback_to_one_connection=False,
+            _return_subsampling_stats=_return_subsampling_stats,
             number_of_batch_rows=number_of_batch_rows,
+            sampling_type=sampling_type,
+            sample_size_limit=sample_size_limit,
             _wml_client=self._wml_client
         )
+
+        iterable_dataset = ExperimentIterableDataset(**experiment_iterable_dataset_setup_parameters)
         data_loader = ExperimentDataLoader(dataset=iterable_dataset)
 
-        for data in data_loader:
-            return data
+        try:
+
+            if not return_data_as_iterator:
+                for data in data_loader:
+                    return data
+            else:
+                return data_loader
+        except TypeError as e1:
+            # note: retry if there is problem with types:
+            # try download data with  infer_as_varchar set to 'true' if some error occurs
+            # final data downloaded and converted to proper types might has smaller size than sample_size_limit,
+            # because data limit is calculated on data downloaded as varchar, before the conversion to more optimal type.
+            try:
+                iterable_dataset = ExperimentIterableDataset(**experiment_iterable_dataset_setup_parameters)
+
+                iterable_dataset.connection.infer_as_varchar = 'true'
+                data_loader = ExperimentDataLoader(dataset=iterable_dataset)
+
+                if not return_data_as_iterator:
+                    for data in data_loader:
+                        return data
+                else:
+                    return data_loader
+            except Exception as e2:
+                raise DataStreamError(f"First attempt of downloading data failed with error: {e1}. \n"
+                                      f"Retry and use infer as varchar also failed with error: {e2}.")
 
     def _upload_data_via_flight_service(self,
                                         data_location: 'DataConnection',
@@ -982,3 +1021,11 @@ class BaseDataConnection(ABC):
             updated_path = remote_name
 
         return updated_path
+
+    @staticmethod
+    def _get_token_from_environment():
+        if os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE'):
+            with open(os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE'), 'r') as f:
+                return f.read()
+        else:
+            return os.environ.get('USER_ACCESS_TOKEN')
