@@ -56,7 +56,8 @@ import numpy as np
 from ibm_watson_machine_learning.utils.autoai.utils import all_logging_disabled, try_import_autoai_libs, \
     try_import_autoai_ts_libs
 from ibm_watson_machine_learning.utils.autoai.watson_studio import get_project
-from ibm_watson_machine_learning.wml_client_error import MissingValue, ApiRequestFailure
+from ibm_watson_machine_learning.data_loaders.datasets.experiment import DEFAULT_SAMPLING_TYPE, DEFAULT_SAMPLE_SIZE_LIMIT
+from ibm_watson_machine_learning.wml_client_error import MissingValue, ApiRequestFailure, WMLClientError
 from .base_connection import BaseConnection
 from .base_data_connection import BaseDataConnection
 from .base_location import BaseLocation
@@ -426,15 +427,15 @@ class DataConnection(BaseDataConnection):
                 learning_type="forecasting",
                 test_size=self.auto_pipeline_params.get('holdout_size'),
                 lookback_window=self.auto_pipeline_params.get('lookback_window'),
-                # supporting_features=None,
+                feature_columns=self.auto_pipeline_params.get('feature_columns'),
                 timestamp_column=self.auto_pipeline_params.get('timestamp_column_name'),
                 # n_jobs=None,
                 # tshirt_size=None,
                 return_only_holdout=False
             )
 
-            X_train = DataFrame(X_train, columns=self.auto_pipeline_params['prediction_columns'])
-            X_holdout = DataFrame(X_holdout, columns=self.auto_pipeline_params['prediction_columns'])
+            X_train = DataFrame(X_train, columns=self.auto_pipeline_params.get('feature_columns', self.auto_pipeline_params['prediction_columns']))
+            X_holdout = DataFrame(X_holdout, columns=self.auto_pipeline_params.get('feature_columns', self.auto_pipeline_params['prediction_columns']))
             y_train = DataFrame(y_train, columns=self.auto_pipeline_params['prediction_columns'])
             y_holdout = DataFrame(y_holdout, columns=self.auto_pipeline_params['prediction_columns'])
 
@@ -521,7 +522,7 @@ class DataConnection(BaseDataConnection):
              number_of_batch_rows: Optional[int] = None,
              **kwargs) -> Union['DataFrame', Tuple['DataFrame', 'DataFrame'], bytes]:
         """
-        Download dataset stored in remote data storage.
+        Download dataset stored in remote data storage. Returns batch up to 1GB.
 
         Parameters
         ----------
@@ -570,7 +571,7 @@ class DataConnection(BaseDataConnection):
         bytes object
 
         Auto holdout split from backend (only train data provided)
-        
+
         Example
         -------
         >>> train_data_connections = optimizer.get_data_connections()
@@ -597,10 +598,19 @@ class DataConnection(BaseDataConnection):
         """
 
         # enables flight automatically for CP4D 4.0.x, 4.5.x and cloud, for 3.0 and 3.5 we do not have a flight service there
-        use_flight = kwargs.get(
-            'use_flight',
-            bool((self._wml_client is not None or 'USER_ACCESS_TOKEN' in os.environ) and (
-                    self._wml_client.ICP_40 or self._wml_client.ICP_45 or self._wml_client.CLOUD_PLATFORM_SPACES)))
+        try:
+            use_flight = kwargs.get(
+                'use_flight',
+                bool((self._wml_client is not None or 'USER_ACCESS_TOKEN' in os.environ or 'RUNTIME_ENV_ACCESS_TOKEN_FILE' in os.environ) and (
+                        self._wml_client.ICP_40 or self._wml_client.ICP_45 or self._wml_client.CLOUD_PLATFORM_SPACES)))
+        except:
+            use_flight = False
+
+        return_data_as_iterator = kwargs.get('return_data_as_iterator', False)
+        sampling_type = kwargs.get('sampling_type', DEFAULT_SAMPLING_TYPE)
+        sample_size_limit = kwargs.get('sample_size_limit', DEFAULT_SAMPLE_SIZE_LIMIT)
+        enable_sampling = kwargs.get('enable_sampling', True)
+        read_normal_data_asset_without_flight = kwargs.get('read_normal_data_asset_without_flight', False)
 
         # Deprecation of excel_sheet as number:
         if isinstance(excel_sheet, int):
@@ -608,7 +618,6 @@ class DataConnection(BaseDataConnection):
                 message="Support for excel sheet as number of the sheet (int) is deprecated! Please set excel sheet with name of the sheet.")
 
         flight_parameters = kwargs.get('flight_parameters', {})
-
         impersonate_header = kwargs.get('impersonate_header', None)
 
         if with_holdout_split and self._user_holdout_exists:  # when this connection is training one
@@ -616,6 +625,9 @@ class DataConnection(BaseDataConnection):
 
         # note: experiment metadata is used only in autogen notebooks
         experiment_metadata = kwargs.get('experiment_metadata')
+        # note: process subsampling stats flag
+        _return_subsampling_stats = kwargs.get("_return_subsampling_stats", False)
+
         if experiment_metadata is not None:
             self.auto_pipeline_params['train_sample_rows_test_size'] = experiment_metadata.get(
                 'train_sample_rows_test_size')
@@ -626,14 +638,16 @@ class DataConnection(BaseDataConnection):
             self.auto_pipeline_params['fairness_info'] = experiment_metadata.get('fairness_info')
             self.auto_pipeline_params['lookback_window'] = experiment_metadata.get('lookback_window')
             self.auto_pipeline_params['timestamp_column_name'] = experiment_metadata.get('timestamp_column_name')
+            self.auto_pipeline_params['feature_columns'] = experiment_metadata.get('feature_columns')
 
             # note: check for cloud
-            if isinstance(experiment_metadata['training_result_reference'].location, (S3Location, AssetLocation)):
-                run_id = experiment_metadata['training_result_reference'].location._training_status.split('/')[-2]
-            # WMLS
-            else:
-                run_id = experiment_metadata['training_result_reference'].location.path.split('/')[-3]
-            self.auto_pipeline_params['run_id'] = run_id
+            if 'training_result_reference' in experiment_metadata:
+                if isinstance(experiment_metadata['training_result_reference'].location, (S3Location, AssetLocation)):
+                    run_id = experiment_metadata['training_result_reference'].location._training_status.split('/')[-2]
+                # WMLS
+                else:
+                    run_id = experiment_metadata['training_result_reference'].location.path.split('/')[-3]
+                self.auto_pipeline_params['run_id'] = run_id
 
             if self._test_data:
                 csv_separator = experiment_metadata.get('test_data_csv_separator', csv_separator)
@@ -681,7 +695,7 @@ class DataConnection(BaseDataConnection):
 
         headers = None
         if self._wml_client is None:
-            token = os.environ.get('USER_ACCESS_TOKEN')
+            token = self._get_token_from_environment()
             if token is not None:
                 headers = {'Authorization': f'Bearer {token}'}
         elif impersonate_header is not None:
@@ -693,80 +707,94 @@ class DataConnection(BaseDataConnection):
                 f"S3 DataConnection is deprecated! Please use data_asset_id instead.")
 
         elif self.type == DataConnectionTypes.DS:
-            if use_flight and not self._obm:
-                from ibm_watson_machine_learning.utils.utils import is_lib_installed
+            # workaround for reading data assets on CPD 4.5: issue #28332
+            data = None
+            if read_normal_data_asset_without_flight and self._wml_client and self._is_data_asset_normal():
+                data = self._download_training_data_from_data_asset_storage()
 
-                is_lib_installed(lib_name='pyarrow', minimum_version='3.0.0', install=True)
+            if data is None:  # read data in normal scenario with flight
+                if use_flight and not self._obm:
+                    from ibm_watson_machine_learning.utils.utils import is_lib_installed
 
-                from pyarrow.flight import FlightError
+                    is_lib_installed(lib_name='pyarrow', minimum_version='3.0.0', install=True)
 
-                _iam_id = None
-                if headers and headers.get('impersonate'):
-                    _iam_id = headers.get('impersonate', {}).get('iam_id')
+                    from pyarrow.flight import FlightError
 
-                self._wml_client._iam_id = _iam_id
+                    _iam_id = None
+                    if headers and headers.get('impersonate'):
+                        _iam_id = headers.get('impersonate', {}).get('iam_id')
 
-                try:
-                    if self._check_if_connection_asset_is_s3():
-                        # note: update flight parameters only if `connection_properties` was not set earlier
-                        #       (e.x. by wml/autoi)
-                        if not flight_parameters.get('connection_properties'):
-                            flight_parameters = self._update_flight_parameters_with_connection_details(flight_parameters)
+                    self._wml_client._iam_id = _iam_id
 
-                    data = self._download_data_from_flight_service(data_location=self,
-                                                                   binary=binary,
-                                                                   read_to_file=read_to_file,
-                                                                   flight_parameters=flight_parameters,
-                                                                   headers=headers,
-                                                                   number_of_batch_rows=number_of_batch_rows)
-
-                except (ConnectionError, FlightError, ApiRequestFailure) as download_data_error:
-                    # note: try to download normal data asset either directly from cams or from mounted NFS
-                    #       to keep backward compatibility
-                    if self._wml_client and (self._is_data_asset_normal() or self._is_data_asset_nfs()):
-                        import warnings
-                        warnings.warn(str(download_data_error), Warning)
-                        data = self._download_training_data_from_data_asset_storage()
-                    else:
-                        raise download_data_error
-
-
-
-            # backward compatibility
-            else:
-                try:
-                    with all_logging_disabled():
+                    try:
                         if self._check_if_connection_asset_is_s3():
-                            cos_client = self._init_cos_client()
+                            # note: update flight parameters only if `connection_properties` was not set earlier
+                            #       (e.x. by wml/autoi)
+                            if not flight_parameters.get('connection_properties'):
+                                flight_parameters = self._update_flight_parameters_with_connection_details(flight_parameters)
 
-                            if self._obm:
-                                data = self._download_obm_data_from_cos(cos_client=cos_client)
-
-                            else:
-                                data = self._download_data_from_cos(cos_client=cos_client)
-                        else:
+                        data = self._download_data_from_flight_service(data_location=self,
+                                                                       binary=binary,
+                                                                       read_to_file=read_to_file,
+                                                                       flight_parameters=flight_parameters,
+                                                                       headers=headers,
+                                                                       enable_sampling=enable_sampling,
+                                                                       sampling_type=sampling_type,
+                                                                       sample_size_limit=sample_size_limit,
+                                                                       number_of_batch_rows=number_of_batch_rows,
+                                                                       return_data_as_iterator=return_data_as_iterator,
+                                                                       _return_subsampling_stats=_return_subsampling_stats)
+                    except (ConnectionError, FlightError, ApiRequestFailure) as download_data_error:
+                        # note: try to download normal data asset either directly from cams or from mounted NFS
+                        #       to keep backward compatibility
+                        if self._wml_client and (self._is_data_asset_normal() or self._is_data_asset_nfs()):
+                            import warnings
+                            warnings.warn(str(download_data_error), Warning)
                             data = self._download_training_data_from_data_asset_storage()
+                        else:
+                            raise download_data_error
 
-                except NotImplementedError as e:
-                    raise e
+                # backward compatibility
+                else:
+                    try:
+                        with all_logging_disabled():
+                            if self._check_if_connection_asset_is_s3():
+                                cos_client = self._init_cos_client()
 
-                except FileNotFoundError as e:
-                    raise e
+                                if self._obm:
+                                    data = self._download_obm_data_from_cos(cos_client=cos_client)
 
-                except Exception as e:
-                    # do not try Flight if we are on the cloud
-                    if self._wml_client is not None:
-                        if not self._wml_client.ICP:
-                            raise e
+                                else:
+                                    data = self._download_data_from_cos(cos_client=cos_client)
+                            else:
+                                data = self._download_training_data_from_data_asset_storage()
 
-                    elif os.environ.get('USER_ACCESS_TOKEN') is None:
-                        raise CannotReadSavedRemoteDataBeforeFit()
+                    except NotImplementedError as e:
+                        raise e
 
-                    data = self._download_data_from_flight_service(data_location=self,
-                                                                   binary=binary,
-                                                                   read_to_file=read_to_file,
-                                                                   flight_parameters=flight_parameters,
-                                                                   headers=headers)
+                    except FileNotFoundError as e:
+                        raise e
+
+                    except Exception as e:
+                        # do not try Flight if we are on the cloud
+                        if self._wml_client is not None:
+                            if not self._wml_client.ICP:
+                                raise e
+
+                        elif os.environ.get('USER_ACCESS_TOKEN') is None and os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE') is None:
+                            raise CannotReadSavedRemoteDataBeforeFit()
+
+                        data = self._download_data_from_flight_service(data_location=self,
+                                                                       binary=binary,
+                                                                       read_to_file=read_to_file,
+                                                                       flight_parameters=flight_parameters,
+                                                                       headers=headers,
+                                                                       enable_sampling=enable_sampling,
+                                                                       sampling_type=sampling_type,
+                                                                       sample_size_limit=sample_size_limit,
+                                                                       number_of_batch_rows=number_of_batch_rows,
+                                                                       return_data_as_iterator=return_data_as_iterator,
+                                                                       _return_subsampling_stats=_return_subsampling_stats)
 
         elif self.type == DataConnectionTypes.FS:
 
@@ -788,7 +816,13 @@ class DataConnection(BaseDataConnection):
                                                                binary=binary,
                                                                read_to_file=read_to_file,
                                                                flight_parameters=flight_parameters,
-                                                               headers=headers)
+                                                               headers=headers,
+                                                               enable_sampling=enable_sampling,
+                                                               sampling_type=sampling_type,
+                                                               sample_size_limit=sample_size_limit,
+                                                               number_of_batch_rows=number_of_batch_rows,
+                                                               return_data_as_iterator=return_data_as_iterator,
+                                                               _return_subsampling_stats=_return_subsampling_stats)
 
             else:  # backward compatibility
                 try:
@@ -815,14 +849,19 @@ class DataConnection(BaseDataConnection):
                         if not self._wml_client.ICP:
                             raise e
 
-                    elif os.environ.get('USER_ACCESS_TOKEN') is None:
+                    elif os.environ.get('USER_ACCESS_TOKEN') is None and os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE') is None:
                         raise CannotReadSavedRemoteDataBeforeFit()
 
                     data = self._download_data_from_flight_service(data_location=self,
                                                                    binary=binary,
                                                                    read_to_file=read_to_file,
                                                                    flight_parameters=flight_parameters,
-                                                                   headers=headers)
+                                                                   headers=headers,
+                                                                   enable_sampling=enable_sampling,
+                                                                   sampling_type=sampling_type,
+                                                                   sample_size_limit=sample_size_limit,
+                                                                   number_of_batch_rows=number_of_batch_rows,
+                                                                   _return_subsampling_stats=_return_subsampling_stats)
 
         if binary:
             return data
@@ -833,19 +872,44 @@ class DataConnection(BaseDataConnection):
 
         else:
             if with_holdout_split:  # when this connection is training one
-                X_train, X_holdout, y_train, y_holdout = self._recreate_holdout(data=data)
-                return X_train, X_holdout, y_train, y_holdout
+                if return_data_as_iterator:
+                    raise WMLClientError("The flags `return_data_as_iterator` and `with_holdout_split` cannot be set both in the same time.")
+
+                if _return_subsampling_stats:
+                    X_train, X_holdout, y_train, y_holdout = self._recreate_holdout(data=data[0])
+                    return X_train, X_holdout, y_train, y_holdout, data[1], data[2]
+                else:
+                    X_train, X_holdout, y_train, y_holdout = self._recreate_holdout(data=data)
+                    return X_train, X_holdout, y_train, y_holdout
 
             else:  # when this data connection is a test / holdout one
-                if self.auto_pipeline_params.get('prediction_columns') or \
-                        not self.auto_pipeline_params.get('prediction_column') or \
-                        (self.auto_pipeline_params.get('prediction_column') and self.auto_pipeline_params.get('prediction_column') not in data.columns):
-                    # timeseries dataset does not have prediction columns. Whole data set is returned:
-                    test_X = data
-                else:
-                    test_X, test_y = self._recreate_holdout(data=data, with_holdout_split=False)
-                    test_X[self.auto_pipeline_params.get('prediction_column', 'prediction_column')] = test_y
-                return test_X  # return one dataframe
+                if return_data_as_iterator:
+                    return data
+
+                if _return_subsampling_stats:
+                    if self.auto_pipeline_params.get('prediction_columns') or \
+                            not self.auto_pipeline_params.get('prediction_column') or \
+                            (self.auto_pipeline_params.get('prediction_column') and self.auto_pipeline_params.get(
+                                'prediction_column') not in data[0].columns):
+                        # timeseries dataset does not have prediction columns. Whole data set is returned:
+                        test_X = data
+                        return test_X
+                    else:
+                        test_X, test_y = self._recreate_holdout(data=data[0], with_holdout_split=False)
+                        test_X[self.auto_pipeline_params.get('prediction_column', 'prediction_column')] = test_y
+                        return test_X, data[1], data[2]
+
+                else: # when this data connection is a test / holdout one and no subsampling stats are needed
+                    if self.auto_pipeline_params.get('prediction_columns') or \
+                            not self.auto_pipeline_params.get('prediction_column') or \
+                            (self.auto_pipeline_params.get('prediction_column') and self.auto_pipeline_params.get(
+                                'prediction_column') not in data.columns):
+                        # timeseries dataset does not have prediction columns. Whole data set is returned:
+                        test_X = data
+                    else:
+                        test_X, test_y = self._recreate_holdout(data=data, with_holdout_split=False)
+                        test_X[self.auto_pipeline_params.get('prediction_column', 'prediction_column')] = test_y
+                    return test_X  # return one dataframe
 
     def write(self, data: Union[str, 'DataFrame'], remote_name: str = None, **kwargs) -> None:
         """
@@ -862,7 +926,7 @@ class DataConnection(BaseDataConnection):
         # enables flight automatically for CP4D 4.0.x and cloud, for 3.0 and 3.5 we do not have a flight service there
         use_flight = kwargs.get(
             'use_flight',
-            bool((self._wml_client is not None or 'USER_ACCESS_TOKEN' in os.environ) and (
+            bool((self._wml_client is not None or 'USER_ACCESS_TOKEN' in os.environ or 'RUNTIME_ENV_ACCESS_TOKEN_FILE' in os.environ) and (
                     self._wml_client.ICP_40 or self._wml_client.ICP_45 or self._wml_client.CLOUD_PLATFORM_SPACES)))
 
         flight_parameters = kwargs.get('flight_parameters', {})
@@ -871,7 +935,7 @@ class DataConnection(BaseDataConnection):
 
         headers = None
         if self._wml_client is None:
-            token = os.environ.get('USER_ACCESS_TOKEN')
+            token = self._get_token_from_environment()
             if token is None:
                 raise ConnectionError("WML client missing. Please initialize WML client and pass it to "
                                       "DataConnection._wml_client property to be able to use this functionality.")
@@ -1297,7 +1361,7 @@ class AssetLocation(BaseLocation):
 
     @classmethod
     def _is_wsd(cls):
-        if os.environ.get('USER_ACCESS_TOKEN'):
+        if os.environ.get('USER_ACCESS_TOKEN') or os.environ.get('RUNTIME_ENV_ACCESS_TOKEN_FILE'):
             return False
 
         try:
@@ -1394,6 +1458,10 @@ class AssetLocation(BaseLocation):
         """Return a json dictionary representing this model."""
         _dict = vars(self).copy()
 
+        if _dict.get('id', False) is None and _dict.get('href'):
+            items = self.href.split('/')
+            _dict['id'] = items[-1].split('?')[0]
+
         del _dict['_wsd']
         del _dict[f"_{self.__class__.__name__}__wml_client"]
 
