@@ -94,6 +94,11 @@ class FlightConnection:
     fallback_to_one_connection: bool, optional
         Indicates if in case of failure we should switch to the one connection and try again.
         Default True.
+
+    return_subsampling_stats: bool, optional
+        Indicates whether return batch data stats: dataset size, no. of batches.
+        Applicable only if subsampling is enabled.
+        Default False.
     """
 
     def __init__(self,
@@ -116,6 +121,7 @@ class FlightConnection:
                  fallback_to_one_connection: Optional[bool] = True,
                  number_of_batch_rows: int = None,
                  stop_after_first_batch: bool = False,
+                 return_subsampling_stats: bool = False,
                  **kwargs
                  ) -> None:
 
@@ -163,6 +169,7 @@ class FlightConnection:
         self.subsampled_data: 'pd.DataFrame' = pd.DataFrame()
         self.data: 'pd.DataFrame' = pd.DataFrame()
         self.enable_subsampling = enable_subsampling
+        self.return_subsampling_stats = return_subsampling_stats
         self.total_size = 0  # total size of downloaded data in Bytes (only in single thread)
         self.downloaded_data_size = 0  # total size of downloaded data in Bytes (every case)
         self.batch_queue = []
@@ -181,6 +188,8 @@ class FlightConnection:
         self.read_binary = False
         self.write_binary = False
 
+        self._infer_as_varchar = 'false' # by default set infer_as_varchar to false in flight command. If None - the infer_as_varchar parameter won't be send.
+
         additional_connection_args = {}
         if os.environ.get('TLS_ROOT_CERTS_PATH'):
             additional_connection_args['tls_root_certs'] = os.environ.get('TLS_ROOT_CERTS_PATH')
@@ -199,6 +208,22 @@ class FlightConnection:
             **additional_connection_args
         )
 
+        # note: client as property and setter for dynamic href creation for AssetLocation
+
+    @property
+    def infer_as_varchar(self):
+        return self._infer_as_varchar
+
+    @infer_as_varchar.setter
+    def infer_as_varchar(self, var):
+        if var is None:
+            self._infer_as_varchar = var
+        elif var in ('true', True, 'false', False):
+            self._infer_as_varchar = str(var).lower()
+        else:
+            raise ValueError("FlightConnection.infer_as_varchar property received invalid value."
+                             "A valid value is one of (None, 'true', 'false')")
+
     def _q_put_nowait(self, item):
         # we are not interested in q size increase, so no Condition waiting
         self.q.put_nowait(item)
@@ -225,9 +250,11 @@ class FlightConnection:
 
     def _set_default_flight_location(self) -> None:
         """Try to set default flight location and port from WS."""
-        if not os.environ.get('FLIGHT_SERVICE_LOCATION') and self._wml_client and self._wml_client.CLOUD_PLATFORM_SPACES:
+        if not os.environ.get(
+                'FLIGHT_SERVICE_LOCATION') and self._wml_client and self._wml_client.CLOUD_PLATFORM_SPACES:
             try:
-                flight_location = self._wml_client.PLATFORM_URLS_MAP[self._wml_client.wml_credentials['url']].replace('https://', '')
+                flight_location = self._wml_client.PLATFORM_URLS_MAP[self._wml_client.wml_credentials['url']].replace(
+                    'https://', '')
             except Exception as e:
                 if self._wml_client.wml_credentials['url'] in self._wml_client.PLATFORM_URLS_MAP.values():
                     flight_location = self._wml_client.wml_credentials['url'].replace('https://', '')
@@ -338,7 +365,8 @@ class FlightConnection:
 
                 elif 'The property [infer_as_varchar] is not supported.' in str(e):
                     # Don't sent infer_as_varchar in flight command and try again.
-                    for source_command in self._select_source_command(_use_infer_as_varchar=False):
+                    self.infer_as_varchar = None
+                    for source_command in self._select_source_command():
                         info = self.flight_client.get_flight_info(
                             flight.FlightDescriptor.for_command(source_command)
                         )
@@ -456,6 +484,7 @@ class FlightConnection:
 
     def _process_data(self, data: pd.DataFrame) -> pd.DataFrame:
         """Process data columns and log data size"""
+        logger.debug(f"Process data: Casting string data to numerical columns")
         data = self._cast_columns_to_float_64(data)
         if len(data) < 100 and self.enable_subsampling:
             raise ValueError(f"We do not have enough rows {len(data)} / minimum 100")
@@ -558,6 +587,10 @@ class FlightConnection:
             logger.debug(f"RT {thread_number}: Finishing thread work...")
             self._q_put_nowait((thread_number, 0))  # finish this thread
 
+        except StopIteration:
+            # Reading of batch finished, result committed."
+            self._q_put_nowait((thread_number, 0))
+
         except Exception as e:
             logger.debug(f"RT {thread_number}: Some error occurred. Error: {e}")
             self._q_put_nowait((thread_number, 0))
@@ -611,6 +644,7 @@ class FlightConnection:
         for n, batch in enumerate(self.create_logical_batch(timeout=60 * 60, _type='data')):
             logger.debug(f"IR: Logical batch number {n} received.")
             logger.debug(f"IR: Passing batch to upper layer.")
+            self._check_for_breaking_exceptions()
             yield batch
             logger.debug(f"IR: We have control back, creating next batch...")
             if n == 0 and self.stop_after_first_batch:
@@ -639,7 +673,6 @@ class FlightConnection:
                 else:
                     normal_read_thread = threading.Thread(target=self.normal_read)
                     normal_read_thread.start()
-                    
 
                 for n, endpoints in enumerate(self.get_endpoints()):
                     self.max_flight_batch_number = len(endpoints)  # when flight does not want to open more endpoints
@@ -666,26 +699,27 @@ class FlightConnection:
 
                 self._q_put_nowait((-1, -1))  # send close queue message (stop reading logical batches)
 
-                if self.enable_subsampling:
-                    subsampling_thread.join()
+                try:
+                    if self.enable_subsampling:
+                        subsampling_thread.join()
 
-                else:
-                    normal_read_thread.join()
+                    else:
+                        normal_read_thread.join()
+                except:
+                    pass
 
         try:
             start_reading_threads()
 
-            # Log every partition thread error (should be the flight service info included)
-            if self.threads_exceptions:
-                logger.debug(f"R: Partitions Thread Errors: {self.threads_exceptions}")
-
-                for msg in self.threads_exceptions:
-                    if "Data could not be read" in msg:
-                        raise TypeError(msg)
+            self._check_for_breaking_exceptions()
 
             if self.enable_subsampling:
                 self.subsampled_data = self._process_data(self.subsampled_data)
-                yield self.subsampled_data
+                if self.return_subsampling_stats:
+                    yield self.subsampled_data, self.downloaded_data_size, self.downloaded_data_size // self.data_batch_size
+                else:
+                    yield self.subsampled_data
+
             else:
                 self.data = self._process_data(self.data)
                 yield self.data
@@ -748,10 +782,12 @@ class FlightConnection:
         logger.debug(f"Starting regression random sampling.")
         max_rows = self._get_max_rows(_type='data')
         downloaded_data_size = 0
+        data_batch_size = []
 
         for data in self.create_logical_batch():
             data_size = sys.getsizeof(data)
             downloaded_data_size += data_size
+            data_batch_size.append(data_size)
             self.callback.status_message(f"Downloaded data size: {downloaded_data_size // 1024 // 1024} MB")
             logger.debug(f"Logical batch size: {data_size}")
 
@@ -768,6 +804,7 @@ class FlightConnection:
             else:
                 self.subsampled_data = data.sample(n=max_rows, random_state=0, axis=0)
 
+            self.data_batch_size = data_batch_size[0]
             logger.debug(f"Subsampled batch size: {sys.getsizeof(self.subsampled_data)}")
 
     def stratified_subsampling(self, label_column: str) -> None:
@@ -775,10 +812,12 @@ class FlightConnection:
         logger.debug(f"Starting classification stratified sampling.")
         max_rows = self._get_max_rows(_type='data')
         downloaded_data_size = 0
+        data_batch_size = []
 
         for data in self.create_logical_batch():
             data_size = sys.getsizeof(data)
             downloaded_data_size += data_size
+            data_batch_size.append(data_size)
             self.callback.status_message(f"Downloaded data size: {downloaded_data_size // 1024 // 1024} MB")
             logger.debug(f"Logical batch size: {data_size}")
 
@@ -806,6 +845,37 @@ class FlightConnection:
                     self.subsampled_data = data.iloc[train_index]
 
             logger.debug(f"Subsampled batch size: {sys.getsizeof(self.subsampled_data)}")
+        self.data_batch_size = data_batch_size[0]
+
+    def truncate_sampling(self, label_column: str) -> None:
+        """Start collecting sampled data (truncate sample for forecasting problem)"""
+        logger.debug(f"Starting forecasting truncate sampling.")
+        max_rows = self._get_max_rows(_type='data')
+        downloaded_data_size = 0
+        data_batch_size = []
+
+        for data in self.create_logical_batch():
+            data_size = sys.getsizeof(data)
+            downloaded_data_size += data_size
+            data_batch_size.append(data_size)
+            self.callback.status_message(f"Downloaded data size: {downloaded_data_size // 1024 // 1024} MB")
+            logger.debug(f"Logical batch size: {data_size}")
+
+            # join previous sampled batch with new one
+            if self.subsampled_data is not None:
+                data = pd.concat([self.subsampled_data, data])
+
+            if len(data) <= max_rows:
+                self.subsampled_data = data
+
+            else:
+                timestamp_column = self.params.get('timestamp_column')
+                if timestamp_column is not None:
+                    data = data.sort_values(timestamp_column)
+                self.subsampled_data = data.tail(max_rows)
+
+            logger.debug(f"Subsampled batch size: {sys.getsizeof(self.subsampled_data)}")
+        self.data_batch_size = data_batch_size[0]
 
     def normal_read(self):
         """Start collecting all the data when user do not want to subsample.
@@ -827,7 +897,8 @@ class FlightConnection:
         while True:
             try:
                 if threads_finished == self.max_flight_batch_number:
-                    yield pd.concat(logical_batch)  # flush last data
+                    data = pd.concat(logical_batch)  # flush last data
+                    yield self._process_data(data)
                     break
 
                 logger.debug("LB: Waiting for mini batch to appear in the queue...")
@@ -838,7 +909,8 @@ class FlightConnection:
 
                 # when the last thread finish reading (finish sequence (-1, -1))
                 if isinstance(data, int) and thread_number == -1 and data == -1:
-                    yield pd.concat(logical_batch)  # flush last data
+                    data = pd.concat(logical_batch)  # flush last data
+                    yield self._process_data(data)
                     break
 
                 # when not the last thread finished reading, continue
@@ -861,13 +933,14 @@ class FlightConnection:
 
             else:
                 logger.debug("LB: Yielding logical batch!")
-                yield pd.concat(logical_batch)
+                data = pd.concat(logical_batch)  # flush last data
+                yield self._process_data(data)
 
                 rows = len(logical_batch[-1])
                 logical_batch = [data.iloc[rows + 1:]]
                 max_rows = self._get_max_rows(_type=_type) - len(logical_batch[-1])
 
-    def _select_source_command(self, infer_schema: bool = True, _use_infer_as_varchar: bool = True) -> List[str]:
+    def _select_source_command(self, infer_schema: bool = True) -> List[str]:
         """Based on a data source type, select appropriate commands for flight service configuration."""
 
         infer_schema = infer_schema
@@ -940,6 +1013,10 @@ class FlightConnection:
                 'interaction_properties'] or self.read_binary):
                 del command['interaction_properties']['encoding']
 
+            if 'quote_character' in command['interaction_properties'] and ('write_mode' in command[
+                'interaction_properties'] or self.read_binary):
+                del command['interaction_properties']['quote_character']
+
             if infer_schema:
                 command['interaction_properties']['infer_schema'] = "true"
             command['interaction_properties']['file_name'] = self.data_location['location']['path']
@@ -955,8 +1032,8 @@ class FlightConnection:
                 command['interaction_properties']['infer_schema'] = "true"
 
         # Property 'infer_as_varchar` needs to be false, when 'infer_schema' is true
-        if command['interaction_properties'].get('infer_schema', "false") == "true" and _use_infer_as_varchar:
-            command['interaction_properties'].update({'infer_as_varchar': "false"})
+        if command['interaction_properties'].get('infer_schema', "false") == "true" and self.infer_as_varchar is not None:
+            command['interaction_properties'].update({'infer_as_varchar': self.infer_as_varchar})
 
         # Git based project assets property
         if self.data_location is not None and self.data_location['location'].get('userfs'):
@@ -1054,21 +1131,36 @@ class FlightConnection:
         writer, reader = self.flight_client.do_put(flight.FlightDescriptor.for_command(commands[0]), schema)
         return writer
 
+    def _check_for_breaking_exceptions(self):
+        # Log every partition thread error (should be the flight service info included)
+        if self.threads_exceptions:
+            logger.debug(f"IR/R: Partitions Thread Errors: {self.threads_exceptions}")
+
+            for msg in self.threads_exceptions:
+                if "Data could not be read" in msg:
+                    raise TypeError(msg)
+                if 'Internal error occurred' in msg:
+                    raise TypeError(msg)
 
     def _get_sampling_strategy(self):
         """
         Return sampling strategy for given sampling and learning type.
         """
         random_sampling_pred_types = (
-            PredictionType.REGRESSION,
+            PredictionType.REGRESSION, PredictionType.CLASSIFICATION, PredictionType.BINARY, PredictionType.MULTICLASS
         )
         stratified_sampling_pred_types = (
             PredictionType.CLASSIFICATION, PredictionType.BINARY, PredictionType.MULTICLASS
         )
+        truncate_sampling_pred_types = (
+            PredictionType.FORECASTING
+        )
 
         if self.sampling_type == SamplingTypes.RANDOM and self.learning_type in random_sampling_pred_types:
             return self.regression_random_sampling
         elif self.sampling_type == SamplingTypes.STRATIFIED and self.learning_type in stratified_sampling_pred_types:
             return self.stratified_subsampling
+        elif self.sampling_type == SamplingTypes.TRUNCATE and self.learning_type in truncate_sampling_pred_types:
+            return self.truncate_sampling
         else:
             raise InvalidSamplingType(self.sampling_type, self.learning_type)
