@@ -358,7 +358,7 @@ class ServiceInstanceNewPlan:
         token_url = self._href_definitions.get_cpd_validation_token_endpoint_href()
         response = requests.get(token_url,
                                 headers={
-                                    'username': 'admin',
+                                    'username': self.get_username(),
                                     'iam-token': iam_token
                                 })
 
