@@ -644,6 +644,7 @@ class ModelCollection:
         import_location =  artifact.meta.prop(MetaNames.IMPORT)
         custom = artifact.meta.prop(MetaNames.CUSTOM)
         metrics = artifact.meta.prop(MetaNames.METRICS)
+        fairness_info = artifact.meta.prop(MetaNames.FAIRNESS_INFO)
         size = artifact.meta.prop(MetaNames.SIZE)
         software_spec = artifact.meta.prop(MetaNames.SOFTWARE_SPEC)
 
@@ -816,177 +817,45 @@ class ModelCollection:
         domain = artifact.meta.prop(MetaNames.DOMAIN)
         project_id = artifact.meta.prop(MetaNames.PROJECT_ID)
         space_id = artifact.meta.prop(MetaNames.SPACE_ID)
-        if artifact.meta.prop(MetaNames.CONTENT_LOCATION) is not None:
-            if isinstance(artifact.meta.prop(MetaNames.CONTENT_LOCATION), str):
-                contentloc = json.loads(artifact.meta.prop(MetaNames.CONTENT_LOCATION))
-            else:
-                contentloc = artifact.meta.prop(MetaNames.CONTENT_LOCATION)
-            content_location = ContentLocation(
-                contentloc.get('url', None),
-                contentloc.get('connection', None),
-                contentloc.get('source', None))
 
-        if artifact.meta.prop(MetaNames.TYPE).startswith("scikit") \
-                or artifact.meta.prop(MetaNames.TYPE).startswith("xgboost"):
-            return MlAssetsCreateModelInput(
-                tags=tags_data_list,
-                space=space,
-                project=project,
-                pipeline=pipeline,
-                type=type_model,
-                name=artifact.name,
-                description=meta.prop(MetaNames.DESCRIPTION),
-                training_data_references=training_data_list,
-                label_column=label_column,
-                hyper_parameters=hyper_param_list,
-                schemas=schemas,
-                domain=domain,
-                custom=custom,
-                runtime=runtime,
-                software_spec=software_spec,
-                model_definition=model_definition,
-                project_id=project_id,
-                space_id=space_id
-            )
-        elif artifact.meta.prop(MetaNames.TYPE).startswith("tensorflow") \
-                or MetaNames.is_archive_framework(artifact.meta.prop(MetaNames.TYPE)):
-            return MlAssetsCreateModelInput(
-                tags=tags_data_list,
-                space=space,
-                project=project,
-                pipeline=pipeline,
-                type=type_model,
-                name=artifact.name,
-                description=meta.prop(MetaNames.DESCRIPTION),
-                training_data_references=training_data_list,
-                label_column=meta.prop(MetaNames.LABEL_FIELD),
-                hyper_parameters=hyper_param_list,
-                schemas=schemas,
-                domain=domain,
-                custom=custom,
-                runtime=runtime,
-                software_spec=software_spec,
-                model_definition=model_definition,
-                project_id=project_id,
-                space_id=space_id
-            )
-        elif artifact.meta.prop(MetaNames.TYPE).startswith("hybrid"):
-            return MlAssetsCreateModelInput(
-                tags=tags_data_list,
-                space=space,
-                project=project,
-                pipeline=pipeline,
-                name=artifact.name,
-                type=type_model,
-                description=meta.prop(MetaNames.DESCRIPTION),
-                training_data_references=training_data_list,
-                label_column=label_column,
-                import_location= import_location,
-                hyper_parameters=hyper_param_list,
-                schemas=schemas,
-                runtime=runtime,
-                custom=custom,
-                domain=domain,
-                software_spec=software_spec,
-                model_definition=model_definition,
-                space_id=space_id,
-                project_id=project_id
-            )
-        else:
-            # if artifact.meta.prop(MetaNames.TRAINING_DATA_REFERENCES) is not None:
-            #
-            #     dataref_list=artifact.meta.prop(MetaNames.TRAINING_DATA_REFERENCES)
-            #     if isinstance(dataref_list, str):
-            #         dataref_list = json.loads(artifact.meta.prop(MetaNames.TRAINING_DATA_REFERENCE))
-            #
-            #     training_data_list = []
-            #     # if isinstance(dataref_list, list):
-            #     #     for iter1 in dataref_list:
-            #     #         training_ref_obj=iter1
-            #     #         name=training_ref_obj.get('name', None)
-            #     #         type=training_ref_obj.get('type', None)
-            #     #         connection = training_ref_obj.get('connection', None)
-            #     #         location = training_ref_obj.get('location', None)
-            #     #         schema = training_ref_obj.get('schema', None)
-            #     #         training_ref = ModelContentLocation(name,type,connection,location,schema)
-            #     #         training_data_list.append(training_ref)
-            #
-            #     if isinstance(dataref_list, dict):
-            #         training_data = ConnectionObjectWithNameRepository(
-            #             dataref_list.get('name', None),
-            #             dataref_list.get('connection', None),
-            #             dataref_list.get('source', None)
-            #         )
-            #         training_data_list.append(training_data)
-            #
-            #     else:
-            #         raise ApiException(404, 'Pipeline not found')
-            if artifact.meta.prop(MetaNames.EVALUATION_METHOD) is not None:
-                metrics = Json2ObjectMapper.read(artifact.meta.prop(MetaNames.EVALUATION_METRICS))
-                metrics = list(map(
-                    lambda metrics_set: EvaluationDefinitionRepositoryMetrics(metrics_set["name"], metrics_set["threshold"], metrics_set["value"]),
-                    metrics
-                ))
-                model_input_evaluation = EvaluationDefinitionRepository(
-                    artifact.meta.prop(MetaNames.EVALUATION_METHOD),
-                    metrics)
+        if artifact.meta.prop(MetaNames.EVALUATION_METHOD) is not None:
+            metrics = Json2ObjectMapper.read(artifact.meta.prop(MetaNames.EVALUATION_METRICS))
+            metrics = list(map(
+                lambda metrics_set: EvaluationDefinitionRepositoryMetrics(metrics_set["name"], metrics_set["threshold"],
+                                                                          metrics_set["value"]),
+                metrics
+            ))
 
-            if artifact.pipeline_artifact() is not None:
-                pipeline_artifact = artifact.pipeline_artifact()
-                model_input = MlAssetsCreateModelInput(
-                    tags=tags_data_list,
-                    space=space,
-                    project=project,
-                    pipeline=pipeline,
-                    type=type_model,
-                    domain=domain,
-                    schema=schemas,
-                    import_location=import_location,
-                    name=artifact.name,
-                    description=meta.prop(MetaNames.DESCRIPTION),
-                    training_definition_url=pipeline_artifact.meta.prop(MetaNames.TRAINING_DEFINITION_VERSION_URL),
-                    label_column=artifact.meta.prop(MetaNames.LABEL_FIELD),
-                    training_data_reference=training_data_list,
-                    input_data_schema=meta.prop(MetaNames.INPUT_DATA_SCHEMA),
-                    evaluation=model_input_evaluation,
-                    training_data_schema=artifact.meta.prop(MetaNames.TRAINING_DATA_SCHEMA),
-                    transformed_label=artifact.meta.prop(MetaNames.TRANSFORMED_LABEL_FIELD),
-                    content_location=content_location,
-                    hyper_parameters=hyper_param_list,
-                    output_data_schema=output_data_schema,
-                    runtime=runtime_input,
-                    software_spec=software_spec,
-                    model_definition = model_definition,
-                    project_id=project_id,
-                    space_id=space_id
-                )
-            else:
-                model_input = MlAssetsCreateModelInput(
-                tags=tags_data_list,
-                name=artifact.name,
-                description=description,
-                transformed_label_column=transformed_label,
-                label_column=label_column,
-                training_data_references=training_data_list,
-                schemas=schemas,
-                hyper_parameters=hyper_param_list,
-                runtime=runtime,
-                space=space,
-                project=project,
-                custom=custom,
-                pipeline=pipeline,
-                training_lib=training_lib,
-                type=type_model,
-                domain = domain,
-                import_location=import_location,
-                metrics = metrics,
-                size = size,
-                software_spec=software_spec,
-                model_definition=model_definition,
-                project_id=project_id,
-                space_id=space_id
-                )
-            return model_input
+        if artifact.meta.prop(MetaNames.FAIRNESS_INFO) and artifact.meta.prop(MetaNames.METRICS):
+            raise ValueError("FAIRNESS_INFO and METRICS cannot be set at the same time.")
+        elif artifact.meta.prop(MetaNames.FAIRNESS_INFO):
+            metrics = [{"context": fairness_info}]
+
+        return MlAssetsCreateModelInput(
+            tags=tags_data_list,
+            space=space,
+            project=project,
+            pipeline=pipeline,
+            type=type_model,
+            training_lib=training_lib,
+            name=artifact.name,
+            description=meta.prop(MetaNames.DESCRIPTION),
+            transformed_label_column=transformed_label,
+            training_data_references=training_data_list,
+            label_column=label_column,
+            import_location=import_location,
+            hyper_parameters=hyper_param_list,
+            schemas=schemas,
+            domain=domain,
+            custom=custom,
+            runtime=runtime,
+            metrics=metrics,
+            size=size,
+            software_spec=software_spec,
+            model_definition=model_definition,
+            project_id=project_id,
+            space_id=space_id
+        )
 
     @staticmethod
     def _get_version_input(artifact):
